import 'package:firebase_storage/firebase_storage.dart';
import 'package:firebase_storage_hooks/firebase_storage_hooks.dart';
import 'package:firebase_storage_hooks/src/models/commit_handler.dart';
import 'package:firebase_storage_hooks/src/models/file_handler.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';

@immutable
class StorageFileHandler<FileType extends Object, ReturnType extends Object> {
  const StorageFileHandler({
    required this.commit,
    required this.fileHandler,
    required this.snapshotListenable,
    required this.remoteFilePath,
  });

  /// This will commit the file(s) to storage. The return value can be used
  /// to revert the changes or to get the result of the commit. If you'd like to
  /// listen to the progress, you can find that in the [snapshot] parameter.
  final Future<CommitHandler<ReturnType>> Function([PlatformFile? platformFile])
      commit;

  /// This will handle selecting and listening to the file(s) that are to be
  /// uploaded.
  final FileHandler<FileType> fileHandler;

  final ValueListenable<AsyncSnapshotWithProgress<ReturnType, TaskSnapshot>>
      snapshotListenable;

  final String? remoteFilePath;
}
